/*************************************************************************
 *                                                                       *
 * This is free software; you can redistribute it and/or modify          *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * at your option) any later version.                                    *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with Foobar; if not, write to:                                  *
 *                   Free Software Foundation, Inc.                      *
 *                   59 Temple Place                                     *
 *                   Suite 330                                           *
 *                   Boston, MA  02111-1307                              *
 *                   USA                                                 *
 *************************************************************************/


#include <stdio.h>
#include <math.h>

struct cfg_struct {
   int shift_code;
   int *key_code;
};

#define MISC_SECTION      "[Misc]"
#define SHIFTED_SECTION   "[Shifted]"
#define UNSHIFTED_SECTION "[Unshifted]"
#define SECTION_CLOSE     "."

static const char *SHIFT_CODE_LABEL  = "ShiftCode";

struct key_struct {
   int def_code;
   char *comment;
   char *def_key;
};

static const char *DEFAULT_CONFIG_FILE = "keycfg.cfg";

#define IPAC_DEFAULT_COUNT  64

/* This shows the names of the 32 I-PAC inputs           */
/* This can be used when generating a config file        */
/* There are 64 entries - the first 32 are for unshifted */
/* The last 32 are for shifted                           */
static const struct key_struct IPAC_DEFAULTS[ IPAC_DEFAULT_COUNT ] = {
/*The first 32 are unshifted inputs*/
   { 0xF5, "1 Up",     "up arrow" },
   { 0xF4, "1 Right",  "right arrow" },
   { 0x14, "1 Bt 1",   "L" },
   { 0xEB, "1 Left",   "Left arrow" },
   { 0x29, "1 Bt 3",   "space" },
   { 0xF2, "1 Down",   "down arrow" },
   { 0x1A, "1 Bt 5",   "z" },
   { 0x11, "1 Bt 2",   "alt" },
   { 0x34, "2 Right",  "g" },
   { 0x12, "1 Bt 4",   "Left shift" },
   { 0x23, "2 Left",   "d" },
   { 0x22, "1 Bt 6",   "x" },
   { 0x2D, "2 Up",     "r" },
   { 0x00, "Auto",     "the code for this MUST be 0x00" },
   { 0x2B, "2 Down",   "f" },
   { 0x00, "JP1",      "the code for this MUST be 0x00" },
   { 0x1C, "2 Bt 1",   "a" },
   { 0x16, "Start 1",  "1" },
   { 0x1B, "2 Bt 2",   "s" },
   { 0x1E, "Start 2",  "2" },
   { 0x15, "2 Bt 3 ",  "q" },
   { 0x2E, "Coin 1 ",  "5" },
   { 0x1D, "2 Bt 4 ",  "w" },
   { 0x36, "Coin 2",   "6" },
   { 0x43, "2 Bt 5",   "i" },
   { 0x21, "1 Bt 7",   "c" },
   { 0x42, "2 Bt 6",   "k" },
   { 0x2A, "1 Bt 8",   "v" },
   { 0x3B, "2 Bt 7",   "j" },
   { 0x00, "AuxClk",   "the code must be 0x00" },
   { 0x00, "AuxData",  "the code must be 0x00" },
   { 0x4B, "2 Bt 8",   "l" },
/*Begin shifted inputs*/
   { 0x0E, "1 Up",     "~ (actually just `)" },
   { 0x0D, "1 Right",  "Tab" },
   { 0x2E, "1 Bt 1",   "5" },
   { 0x5A, "1 Left",   "Enter" },
   { 0x00, "1 Bt 3",   "" },
   { 0x4D, "1 Down",   "p" },
   { 0x00, "1 Bt 5",   "" },
   { 0x00, "1 Bt 2",   "" },
   { 0x00, "2 Right",  "" },
   { 0x00, "1 Bt 4",   "" },
   { 0x00, "2 Left",   "" },
   { 0x00, "1 Bt 6",   "" },
   { 0x00, "2 Up",     "" },
   { 0x00, "Auto",     "the code for this MUST be 0x00" },
   { 0x00, "2 Down",   "" },
   { 0x00, "JP1",      "the code for this MUST be 0x00" },
   { 0x00, "2 Bt 1",   "" },
   { 0x16, "Start 1",  "" },
   { 0x00, "2 Bt 2",   "" },
   { 0x76, "Start 2",  "Esc" },
   { 0x00, "2 Bt 3",   "" },
   { 0x00, "Coin 1",   "" },
   { 0x00, "2 Bt 4",   "" },
   { 0x00, "Coin 2",   "" },
   { 0x00, "2 Bt 5",   "" },
   { 0x00, "1 Bt 7",   "" },
   { 0x00, "2 Bt 6",   "" },
   { 0x00, "1 Bt 8",   "" },
   { 0x00, "2 Bt 7",   "" },
   { 0x00, "AuxClk",   "the code must be 0x00" },
   { 0x00, "AuxData",  "the code must be 0x00" },
   { 0x35, "2 Bt 8",   "Y(for prog)" }
};

static const struct key_struct DEF_SHIFT = {
   0x12, "Start 1", ""  };


#define FULL_KEY_COUNT  104

static const struct key_struct FULL_KEY_LIST[ FULL_KEY_COUNT ] = {
	{ 0x1C, "", "A" },
	{ 0x32, "", "B" },
	{ 0x21, "", "C" },
	{ 0x23, "", "D" },
	{ 0x24, "", "E" },
	{ 0x2B, "", "F" },
	{ 0x34, "", "G" },
	{ 0x33, "", "H" },
	{ 0x43, "", "I" },
	{ 0x3B, "", "J" },
	{ 0x42, "", "K" },
	{ 0x4B, "", "L" },
	{ 0x3A, "", "M" },
	{ 0x31, "", "N" },
	{ 0x44, "", "O" },
	{ 0x4D, "", "P" },
	{ 0x15, "", "Q" },
	{ 0x2D, "", "R" },
	{ 0x1B, "", "S" },
	{ 0x2C, "", "T" },
	{ 0x3C, "", "U" },
	{ 0x2A, "", "V" },
	{ 0x1D, "", "W" },
	{ 0x22, "", "X" },
	{ 0x35, "", "Y" },
	{ 0x1A, "", "Z" },
	{ 0x45, "", "0" },
	{ 0x16, "", "1" },
	{ 0x1E, "", "2" },
	{ 0x26, "", "3" },
	{ 0x25, "", "4" },
	{ 0x2E, "", "5" },
	{ 0x36, "", "6" },
	{ 0x3D, "", "7" },
	{ 0x3E, "", "8" },
	{ 0x46, "", "9" },
	{ 0x0E, "", "`" },
	{ 0x4E, "", "-" },
	{ 0x55, "", "=" },
	{ 0x41, "", "," },
	{ 0x5D, "", "\\" },
	{ 0x66, "", "BKSP" },
	{ 0x29, "", "SPACE" },
	{ 0x0D, "", "TAB" },
	{ 0x58, "", "CAPS" },
	{ 0x12, "", "L SHFT" },
	{ 0x14, "", "L CTRL" },
	{ 0xE0, "", "L GUI" },
	{ 0x11, "", "L ALT" },
	{ 0x59, "", "R SHFT" },
	{ 0xE0, "", "R CTRL" },
	{ 0xE0, "", "R GUI" },
	{ 0xE0, "", "R ALT" },
	{ 0xE0, "", "APPS" },
	{ 0x5A, "", "ENTER" },
	{ 0x76, "", "ESC" },
	{ 0x05, "", "F1" },
	{ 0x06, "", "F2" },
	{ 0x04, "", "F3" },
	{ 0x0C, "", "F4" },
	{ 0x03, "", "F5" },
	{ 0x0B, "", "F6" },
	{ 0x83, "", "F7" },
	{ 0x0A, "", "F8" },
	{ 0x01, "", "F9" },
	{ 0x09, "", "F10" },
	{ 0x78, "", "F11" },
	{ 0x07, "", "F12" },
	{ 0xE0, "", "PRNT  SCRN" },
	{ 0x7E, "", "SCROLL" },
	{ 0xE1, "", "PAUSE" },
	{ 0x54, "", "[" },
	{ 0xE0, "", "INSERT" },
	{ 0xE0, "", "HOME" },
	{ 0xE0, "", "PG UP" },
	{ 0xE0, "", "DELETE" },
	{ 0xE0, "", "END" },
	{ 0xE0, "", "PG DN" },
	{ 0xE0, "", "U ARROW" },
	{ 0xE0, "", "L ARROW" },
	{ 0xE0, "", "D ARROW" },
	{ 0xE0, "", "R ARROW" },
	{ 0x77, "", "NUM" },
	{ 0xE0, "", "KP /" },
	{ 0x7C, "", "KP *" },
	{ 0x7B, "", "KP -" },
	{ 0x79, "", "KP +" },
	{ 0xE0, "", "KP EN" },
	{ 0x71, "", "KP ." },
	{ 0x70, "", "KP 0" },
	{ 0x69, "", "KP 1" },
	{ 0x72, "", "KP 2" },
	{ 0x7A, "", "KP 3" },
	{ 0x6B, "", "KP 4" },
	{ 0x73, "", "KP 5" },
	{ 0x74, "", "KP 6" },
	{ 0x6C, "", "KP 7" },
	{ 0x75, "", "KP 8" },
	{ 0x7D, "", "KP 9" },
	{ 0x5B, "", "]" },
	{ 0x4C, "", ";" },
	{ 0x52, "", "'" },
	{ 0x49, "", "." },
	{ 0x4A, "", "/" }
};


int htoi(char *hexstr);
  
int ReadCfg(char *cfg_file, struct cfg_struct *cfg_data);

int GotoSection(FILE *stream, char *section);

int ReadNextLine(FILE *stream, char *buf_line);

int WriteDefaultCfg(char *filename);

int WriteCfgFile(char *filename, struct key_struct *data_def, 
                 struct key_struct shift_def);

