/*************************************************************************
 * TLV 02/14/2002                                                        *
 * Copyright (C) 2002 Travis Veldkamp                                    *
 *                                                                       *
 * This program is for programming the I-PAC keyboard encoder            *
 * For more information, see http://www.ultimarc.com                     *
 *                                                                       *
 * This is free software; you can redistribute it and/or modify          *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation; either version 2 of the License, or     *
 * at your option) any later version.                                    *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with Foobar; if not, write to:                                  *
 *                   Free Software Foundation, Inc.                      *
 *                   59 Temple Place                                     *
 *                   Suite 330                                           *
 *                   Boston, MA  02111-1307                              *
 *                   USA                                                 *
 *************************************************************************/


#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/types.h>
#include <fcntl.h>

static const char *console   = "/dev/console";
static const char *STDIN_DEV = "/dev/stdin";

/* Possible return codes for entering programming mode */
#define PGM_MODE_OK            0
#define PGM_MODE_FAIL_JUMPER   1
#define PGM_MODE_FAIL_MISC     2
#define PGM_MODE_FAIL_TIMEOUT  3

/* Possible return codes for the programming step */
#define PGM_OK             0
#define PGM_FAIL_CHECKSUM  1
#define PGM_FAIL_MISC      2
#define PGM_FAIL_TIMEOUT   3


#define PROGRAM_BOARD_1	0
#define PROGRAM_BOARD_2 1

#define IPAC_RSP_PGM_OK       'y'
#define IPAC_RSP_JUMPER_ERR   'j'
#define IPAC_RSP_CHECKSUM_ERR 'c'

static const int IPAC_MACRO_MODE  = 0xFE;
static const int IPAC_MAKE_BOARD1 = 0xFD;
static const int IPAC_MAKE_BOARD2 = 0xFE;
static const int IPAC_NO_SHIFT    = 0xFF;


/* Some keys, such as the arrows actually have 2 byte *
 * scan codes.  For example, up is 0xE075. However,   *
 * the I-PAC only accepts 1 byte per to program each  *
 * code.  So, to specify adding 0xE0, we set the MSB  *
 * in the program code.  For example, up the becomes  *
 * 0xF5 instead of 0x75                               *
 * To use the mask, just or it with the 2nd byte of   *
 * any 2 byte key code that starts with 0xE0          *
 * For more information on scan codes, see:           *
 * http://www.barcodeman.com/altek/mule/scandoc.php3  */
static const int IPAC_SEND_E0_MASK = 0x80;

/* There are 64 key codes for the I-PAC - 32 shifted  *
 * and 32 not shifted (some are forced null, but must *
 * be sent regardless                                 */
static const int IPAC_NUM_KEYCODES = 64;

/* inter send delay is 500us = .5ms */
static const int INTER_SEND_DELAY= 500;
static const int MS_DELAY_50 = 50000;

/* Used for IPACDetect() */
static const int DETECT_NONE    = 0;
static const int DETECT_BOARD_1 = 1;
static const int DETECT_BOARD_2 = 2;

/* Used for ProgramIPAC() when IPACDetect() calls it*/
static const int FORCE_PGM_FAIL = 1;
static const int NORMAL_PROGRAM = 0;



int SendByteToIPAC(int console_fd, char byte);

int SetIPACToProgramMode(int console_fd, int mode, int verbosity);

int IPACDetect(int console_fd, int verbosity);

int ProgramIPAC(int console_fd, int *data_array, int shift_code, int verbosity, 
                int force_bad);
